module send_snapshot_m

  implicit none

  private send_ssh_contour

contains

  subroutine send_snapshot(s, dest, tag)

    ! Libraries:
    use ezmpi, only: ezmpi_send

    use derived_types, only: snapshot

    type(snapshot), intent(in):: s
    integer, intent(in):: dest ! rank of receiving MPI process
    integer, intent(in):: tag ! date index for sent snapshot

    ! Local:
    integer i

    !------------------------------------------------------------------

    call ezmpi_send(s%number_vis_extr, dest, tag)

    do i = 1, s%number_vis_extr
       call ezmpi_send(s%list_vis(i)%coord_extr, dest, tag)
       call ezmpi_send(s%list_vis(i)%ssh_extr, dest, tag)
       call ezmpi_send(s%list_vis(i)%cyclone, dest, tag)
       call send_ssh_contour(s%list_vis(i)%out_cont, dest, tag)
       call send_ssh_contour(s%list_vis(i)%speed_cont, dest, tag)
       call ezmpi_send(s%list_vis(i)%max_speed, dest, tag)
       call ezmpi_send(s%list_vis(i)%valid, dest, tag)
       call ezmpi_send(s%list_vis(i)%delta_in, dest, tag)
       call ezmpi_send(s%list_vis(i)%delta_out, dest, tag)
       call ezmpi_send(s%list_vis(i)%radius4, dest, tag)
    end do

    call ezmpi_send(s%extr_map, dest, tag)
    call ezmpi_send(s%ind_extr, dest, tag)

  end subroutine send_snapshot

  !*******************************************************************

  subroutine send_ssh_contour(cont, dest, tag)

    ! Libraries:
    use ezmpi, only: ezmpi_send
    
    use derived_types, only: ssh_contour

    type(ssh_contour), intent(in):: cont
    integer, intent(in):: dest, tag

    !------------------------------------------------------------------

    call ezmpi_send(cont%n_points, dest, tag)
    call ezmpi_send(cont%closed, dest, tag)
    call ezmpi_send(cont%points, dest, tag)
    call ezmpi_send(cont%ssh, dest, tag)
    call ezmpi_send(cont%area, dest, tag)

  end subroutine send_ssh_contour

end module send_snapshot_m
